-- Sistemas Integrados TEKNISA
-- Faturamento de Servios
--
-- Nome: 0000_FASFUNO_LINHA_PRODUTO_FAT.sql
--
-- Descrio:
-- Retorna informaes sobre as linhas de produto a serem faturadas pela Teknisa.
--
-- Parmetros:
-- P_CDCLIENTE - Cdigo do cliente
--
-- Exemplo de utilizao
-- SELECT * 
--   FROM TABLE (LINHA_PRODUTO_FAT (P_CDCLIENTE))
--  ORDER BY DECODE(IDTIPLINPC, 'M', 1, 'L', 1, 'P', 2, 'A', 2, NULL, 0), NMLINPCLIE
--
-- Criao:
-- 15/07/2010 Ederson Melquades da Cunha 

CREATE TYPE LINHA_PRODUTO_FAT_TYPE AS OBJECT 
   (CDCLIENTE VARCHAR2(21),
    CDLINPCLIE VARCHAR2(20),
    CDLINPCLIEINFOR VARCHAR2(10),    
    NMLINPCLIE VARCHAR2(50),
    IDTIPLINPC VARCHAR2(1),
    NRUSUALINPC FLOAT,
    IDMPROCLINPC VARCHAR2(1),
    IDLPROCLINPC VARCHAR2(1),
    VRLICENCUSOLINP FLOAT,
    QTMINIMAFATLINP FLOAT,
    VRMENSALIDALINP FLOAT,
    CDGRUPONFLINPC VARCHAR2(2),
    QTSISTEMAS FLOAT,
    QTATIVAS FLOAT,
    QTVALIDAS FLOAT,
    DTHOJE DATE, 
    DTIVIGVRLINP DATE,
    DTFVIGVRLINP DATE)
/

CREATE TYPE LINP_FAT_TABLE AS TABLE OF LINHA_PRODUTO_FAT_TYPE
/

CREATE OR REPLACE 
FUNCTION LINHA_PRODUTO_FAT 
  (P_CDCLIENTE IN CLIENTE.CDCLIENTE%TYPE,
   P_DATA IN DATE DEFAULT SYSDATE) 
RETURN LINP_FAT_TABLE PIPELINED IS
   OUT_RECORD LINHA_PRODUTO_FAT_TYPE := LINHA_PRODUTO_FAT_TYPE (NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
   CURSOR C_LINP IS 
      (SELECT CLP.CDCLIENTE, CLP.CDLINPCLIE, CLP.CDLINPCLIEINFOR, CLP.NMLINPCLIE,
              CLP.IDTIPLINPC, CLP.NRUSUALINPC, CLP.IDMPROCLINPC,
              CLP.IDLPROCLINPC, ROUND(NVL(VLR.VRLICENCUSOLINP, 0), 2) AS VRLICENCUSOLINP,
              CLP.QTMINIMAFATLINP, ROUND(NVL(VLR.VRMENSALIDALINP, 0), 2) AS VRMENSALIDALINP, CLP.CDGRUPONFLINPC,
              NVL(QTS.QTSISTEMAS, 0) QTSISTEMAS, NVL(QTA.QTATIVAS, 0) QTATIVAS, NVL(QTV.QTVALIDAS, 0) QTVALIDAS,
              TO_CHAR(SYSDATE, 'FMDS') AS DTHOJE, VLR.DTIVIGVRLINP, VLR.DTFVIGVRLINP
         FROM (SELECT CDCLIENTE, CDLINPCLIE, CDLINPCLIEINFOR, NMLINPCLIE,
                      IDTIPLINPC, NRUSUALINPC, IDMPROCLINPC,
                      IDLPROCLINPC, QTMINIMAFATLINP, CDGRUPONFLINPC       
                 FROM CONTRATOLINP
                WHERE CDCLIENTE = P_CDCLIENTE
                  AND DTCONTLINPRESC IS NULL
                UNION ALL
               SELECT P_CDCLIENTE, 'XXX', NULL, 'Sem linha de produto', 
                      NULL, NULL, NULL, NULL, NULL, NULL
                 FROM DUAL
                WHERE EXISTS (SELECT LU.*
                                FROM LICENCAUSO LU
                               WHERE LU.CDCLIENTE = P_CDCLIENTE
                                 AND LU.IDLUSOATIV = 'S'
                                 AND NOT EXISTS (SELECT *
                                                   FROM LINPLUSO LL
                                                  WHERE LL.CDCLIENTE = LU.CDCLIENTE
                                                    AND LL.NRLICENCAUSO = LU.NRLICENCAUSO))) CLP,
              (SELECT SI.CDCLIENTE, SI.CDLINPCLIE, COUNT(*) AS QTSISTEMAS
                 FROM SISTCONTLINP SI
                WHERE SI.CDCLIENTE = P_CDCLIENTE
                GROUP BY SI.CDCLIENTE, SI.CDLINPCLIE) QTS,
              (SELECT LU.CDCLIENTE, LL.CDLINPCLIE, COUNT(*) AS QTATIVAS
                 FROM LINPLUSO LL, CONTRATOLINP CL, LICENCAUSO LU
                WHERE LU.CDCLIENTE = P_CDCLIENTE
                  AND LL.CDCLIENTE = LU.CDCLIENTE
                  AND LL.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CL.CDCLIENTE = LL.CDCLIENTE
                  AND CL.CDLINPCLIE = LL.CDLINPCLIE
                  AND CL.IDTIPLINPC IN ('M', 'L')
                  AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, P_DATA, 'ATIVA') = 'ATIVA'
                GROUP BY LU.CDCLIENTE, LL.CDLINPCLIE
               UNION ALL
               SELECT LU.CDCLIENTE, LL.CDLINPCLIE, COUNT(*) AS QTATIVAS
                 FROM LINPLUSO LL, CAIXALUSO CX, CONTRATOLINP CL, LICENCAUSO LU
                WHERE LU.CDCLIENTE = P_CDCLIENTE
                  AND LL.CDCLIENTE = LU.CDCLIENTE
                  AND LL.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CX.CDCLIENTE = LU.CDCLIENTE
                  AND CX.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CL.CDCLIENTE = LL.CDCLIENTE
                  AND CL.CDLINPCLIE = LL.CDLINPCLIE
                  AND CL.IDTIPLINPC IN ('P', 'A')
                  AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, P_DATA, 'ATIVA') = 'ATIVA'
                GROUP BY LU.CDCLIENTE, LL.CDLINPCLIE
               UNION ALL
               SELECT LU.CDCLIENTE, 'XXX' AS CDLINPCLIE, COUNT(*) AS QTATIVAS
                 FROM LICENCAUSO LU
                WHERE LU.CDCLIENTE = P_CDCLIENTE
                  AND LU.IDLUSOATIV = 'S'
                  AND NOT EXISTS (SELECT *
                                    FROM LINPLUSO LL
                                   WHERE LL.CDCLIENTE = LU.CDCLIENTE
                                     AND LL.NRLICENCAUSO = LU.NRLICENCAUSO)
                GROUP BY LU.CDCLIENTE, 2) QTA,
              (SELECT LU.CDCLIENTE, LL.CDLINPCLIE, COUNT(*) AS QTVALIDAS
                 FROM LINPLUSO LL, CONTRATOLINP CL, LICENCAUSO LU
                WHERE LU.CDCLIENTE = P_CDCLIENTE
                  AND LL.CDCLIENTE = LU.CDCLIENTE
                  AND LL.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CL.CDCLIENTE = LL.CDCLIENTE
                  AND CL.CDLINPCLIE = LL.CDLINPCLIE
                  AND CL.IDTIPLINPC IN ('M', 'L')
                  AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA, P_DATA, 'VALIDA') = 'VALIDA'
                GROUP BY LU.CDCLIENTE, LL.CDLINPCLIE
               UNION ALL
               SELECT LU.CDCLIENTE, LL.CDLINPCLIE, COUNT(*) AS QTVALIDAS
                 FROM LINPLUSO LL, CAIXALUSO CX, CONTRATOLINP CL, LICENCAUSO LU
                WHERE LU.CDCLIENTE = P_CDCLIENTE
                  AND LL.CDCLIENTE = LU.CDCLIENTE
                  AND LL.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CX.CDCLIENTE = LU.CDCLIENTE
                  AND CX.NRLICENCAUSO = LU.NRLICENCAUSO
                  AND CL.CDCLIENTE = LL.CDCLIENTE
                  AND CL.CDLINPCLIE = LL.CDLINPCLIE
                  AND CL.IDTIPLINPC IN ('P', 'A')
                  AND SITUACAO_LINPLUSO (LU.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, P_DATA, 'VALIDA') = 'VALIDA'
                GROUP BY LU.CDCLIENTE, LL.CDLINPCLIE) QTV,
              (SELECT VL.CDCLIENTE, VL.CDLINPCLIE, VL.VRLICENCUSOLINP,
                      VL.VRMENSALIDALINP, VL.DTFVIGVRLINP, VL.DTIVIGVRLINP
                 FROM VALORLINP VL
                WHERE VL.CDCLIENTE = P_CDCLIENTE
                  AND VL.DTIVIGVRLINP = (SELECT MAX(DTIVIGVRLINP)
                                           FROM VALORLINP VI
                                          WHERE VI.CDCLIENTE  = VL.CDCLIENTE
                                            AND VI.CDLINPCLIE = VL.CDLINPCLIE)) VLR
        WHERE QTS.CDCLIENTE (+) = CLP.CDCLIENTE
          AND QTS.CDLINPCLIE (+) = CLP.CDLINPCLIE
          AND QTA.CDCLIENTE (+) = CLP.CDCLIENTE
          AND QTA.CDLINPCLIE (+) = CLP.CDLINPCLIE
          AND QTV.CDCLIENTE (+) = CLP.CDCLIENTE
          AND QTV.CDLINPCLIE (+) = CLP.CDLINPCLIE
          AND VLR.CDCLIENTE (+) = CLP.CDCLIENTE
          AND VLR.CDLINPCLIE (+) = CLP.CDLINPCLIE);
BEGIN
   OPEN C_LINP;
   LOOP
      FETCH C_LINP INTO 
         OUT_RECORD.CDCLIENTE,
         OUT_RECORD.CDLINPCLIE,
         OUT_RECORD.CDLINPCLIEINFOR,
         OUT_RECORD.NMLINPCLIE,
         OUT_RECORD.IDTIPLINPC,
         OUT_RECORD.NRUSUALINPC,
         OUT_RECORD.IDMPROCLINPC,
         OUT_RECORD.IDLPROCLINPC,
         OUT_RECORD.VRLICENCUSOLINP,
         OUT_RECORD.QTMINIMAFATLINP,
         OUT_RECORD.VRMENSALIDALINP,
         OUT_RECORD.CDGRUPONFLINPC,
         OUT_RECORD.QTSISTEMAS,
         OUT_RECORD.QTATIVAS,
         OUT_RECORD.QTVALIDAS,
         OUT_RECORD.DTHOJE,
         OUT_RECORD.DTIVIGVRLINP,
         OUT_RECORD.DTFVIGVRLINP;
      EXIT WHEN C_LINP%NOTFOUND;   
      PIPE ROW (OUT_RECORD);
   END LOOP;
   CLOSE C_LINP;
   RETURN;
END;
/
